@@ -1,18 +1,20 @@
 __author__ = 'Lene Preuss <lene.preuss@gmail.com>'
 
 import logging
+from argparse import Namespace
 from functools import lru_cache
 from hashlib import sha256
 from pathlib import Path
-from subprocess import call  # noqa: S404
-from typing import Any, Callable, Dict, Tuple, List, Optional
+from shlex import quote
+from subprocess import call  # nosec
+from typing import Any, Callable, Dict, List, Optional
 
 import imagehash
 
 from duplicate_images.common import path_with_parent
-from duplicate_images.function_types import ActionFunction, HashFunction
+from duplicate_images.function_types import ActionFunction, HashFunction, ImagePair
 
-__all__ = ['call', 'quote', 'get_hash_size_kwargs', 'IMAGE_HASH_ALGORITHM', 'ACTIONS_ON_EQUALITY']
+__all__ = ['call', 'get_hash_size_kwargs', 'IMAGE_HASH_ALGORITHM', 'ACTIONS_ON_EQUALITY']
 
 
 @lru_cache(maxsize=None)
@@ -30,7 +32,7 @@ def compare_exactly(file: Path, other_file: Path) -> bool:
     return get_size(other_file) == get_size(file) and get_hash(file) == get_hash(other_file)
 
 
-def ascending_by_size(pair: Tuple[Path, Path]) -> List[Path]:
+def ascending_by_size(pair: ImagePair) -> List[Path]:
     return sorted(pair, key=lambda path: path.stat().st_size)
 
 
@@ -39,18 +41,11 @@ def delete_with_log_message(file: Path) -> None:
     logging.info('Deleted %s', path_with_parent(file))
 
 
-def quote(string: str) -> str:
-    if '"' in string:
-        if "'" in string:
-            raise ValueError(f'{string} contains both single and double quotes, giving up')
-        quotes = "'"
-    else:
-        quotes = '"'
-    return f'{quotes}{string}{quotes}'
-
-
-def quote_print(pair: Tuple[Path, Path]) -> None:
-    print(f'{quote(str(pair[0]))} {quote(str(pair[1]))}')
+def shell_exec(args: Namespace, pair: ImagePair) -> None:
+    cmd = args.exec
+    for num, path in enumerate(pair):
+        cmd = cmd.replace(f"{'{'}{num+1}{'}'}", f'"{path}"')
+    call(cmd, shell=True)  # nosec
 
 
 def get_hash_size_kwargs(algorithm: HashFunction, size: Optional[int]) -> Dict[str, int]:
@@ -81,17 +76,22 @@ ALGORITHM_DEFAULTS = {
 }
 
 ACTIONS_ON_EQUALITY: Dict[str, ActionFunction] = {
-    'delete-first': lambda pair: delete_with_log_message(pair[0]),
-    'd1': lambda pair: delete_with_log_message(pair[0]),
-    'delete-second': lambda pair: delete_with_log_message(pair[1]),
-    'd2': lambda pair: delete_with_log_message(pair[1]),
-    'delete-bigger': lambda pair: delete_with_log_message(ascending_by_size(pair)[-1]),
-    'd>': lambda pair: delete_with_log_message(ascending_by_size(pair)[-1]),
-    'delete-smaller': lambda pair: delete_with_log_message(ascending_by_size(pair)[0]),
-    'd<': lambda pair: delete_with_log_message(ascending_by_size(pair)[0]),
-    'eog': lambda pair: call(['eog'] + [str(pic) for pic in pair]),  # noqa: S603
-    'xv': lambda pair: call(['xv', '-nolim'] + [str(pic) for pic in pair]),  # noqa: S603
-    'print': lambda pair: print(pair[0], pair[1]),
-    'quote': quote_print,
-    'none': lambda pair: None
+    'delete-first': lambda args, pair: delete_with_log_message(pair[0]),
+    'd1': lambda args, pair: delete_with_log_message(pair[0]),
+    'delete-second': lambda args, pair: delete_with_log_message(pair[1]),
+    'd2': lambda args, pair: delete_with_log_message(pair[1]),
+    'delete-bigger': lambda args, pair: delete_with_log_message(ascending_by_size(pair)[-1]),
+    'd>': lambda args, pair: delete_with_log_message(ascending_by_size(pair)[-1]),
+    'delete-smaller': lambda args, pair: delete_with_log_message(ascending_by_size(pair)[0]),
+    'd<': lambda args, pair: delete_with_log_message(ascending_by_size(pair)[0]),
+    'eog': lambda args, pair: call(['eog'] + [str(pic) for pic in pair]),  # nosec
+    'xv': lambda args, pair: call(['xv', '-nolim'] + [str(pic) for pic in pair]),  # nosec
+    'print': lambda args, pair: print(pair[0], pair[1]),
+    'print_inline': lambda args, pair: print(pair[0], pair[1], end=' '),
+    'quote': lambda args, pair: print(f'{quote(str(pair[0]))} {quote(str(pair[1]))}'),
+    'quote_inline': lambda args, pair: print(
+        f'{quote(str(pair[0]))} {quote(str(pair[1]))}', end=' '
+    ),
+    'exec': lambda args, pair: shell_exec(args, pair),  # pylint: disable=unnecessary-lambda
+    'none': lambda args, pair: None
 }
